@@ -41,8 +41,15 @@ public class EcmaDoc {
 	
 	public static string GetClassName (XmlDocument doc)
 	{
-		string name = doc.SelectSingleNode ("/Type").Attributes ["Name"].InnerText;
-		return name.Replace("+", ".");
+		return GetDisplayName (doc.SelectSingleNode ("/Type")).Replace ("+", ".");
+	}
+
+	public static string GetDisplayName (XmlNode type)
+	{
+		if (type.Attributes ["DisplayName"] != null) {
+			return type.Attributes ["DisplayName"].Value;
+		}
+		return type.Attributes ["Name"].Value;
 	}
 
 	public static string GetClassAssembly (XmlDocument doc)
@@ -372,8 +379,6 @@ public class EcmaProvider : Provider {
 		}
 	}
 
-	static NodeCompare NodeComparer = new NodeCompare ();
-
 	string GetMemberName (XmlNode node)
 	{
 		return node.Attributes ["MemberName"].InnerText;
@@ -526,9 +531,6 @@ public class EcmaHelpSource : HelpSource {
 		if (type_node.Nodes == null)
 			return null;
 
-		string lasturl = "";
-		XmlNode doc = null;
-
 		string membername = member;
 		string[] argtypes = null;
 		if (member.IndexOf("(") > 0) {
@@ -702,13 +704,12 @@ public class EcmaHelpSource : HelpSource {
 	
 	public string GetUrlForType (string prefix, string ns, string type, string member, out Node match_node)
 	{
-		if (!prefix.EndsWith(":")) throw new ArgumentException("prefix");
+		if (!prefix.EndsWith(":"))
+			throw new ArgumentException("prefix");
 			
 		// If a nested type, compare only inner type name to node list.
 		// This should be removed when the node list doesn't lose the containing type name.
-		type = type.Replace("+", ".");
-
-		string nsp = prefix + ns;
+		type = ToEscapedTypeName (type.Replace("+", "."));
 
 		foreach (Node ns_node in Tree.Nodes){
 			string ns_node_namespace = ns_node.Element.Substring (2);
@@ -722,6 +723,9 @@ public class EcmaHelpSource : HelpSource {
 				string cname;
 				if (element.StartsWith("T:")) {
 					cname = element.Substring(2);
+					string _ns;
+					RootTree.GetNamespaceAndType (cname, out _ns, out cname);
+					cname = ToEscapedTypeName (cname);
 					int pidx = cname.LastIndexOf (".");
 					cname = cname.Substring(pidx+1);
 					pidx = cname.LastIndexOf ("/");
@@ -773,6 +777,41 @@ public class EcmaHelpSource : HelpSource {
 		return null;
 	}
 	
+	public static string ToEscapedTypeName (string typename)
+	{
+		StringBuilder filename = new StringBuilder (typename.Length);
+		int numArgs = 0;
+		int numLt = 0;
+		bool copy = true;
+
+		for (int i = 0; i < typename.Length; ++i) {
+			char c = typename [i];
+			switch (c) {
+				case '<':
+					copy = false;
+					++numLt;
+					break;
+				case '>':
+					--numLt;
+					if (numLt == 0) {
+						filename.Append ('`').Append ((numArgs+1).ToString());
+						numArgs = 0;
+						copy = true;
+					}
+					break;
+				case ',':
+					if (numLt == 1)
+						++numArgs;
+					break;
+				default:
+					if (copy)
+						filename.Append (c);
+					break;
+			}
+		}
+		return filename.ToString ();
+	}
+	
 	public override string GetNodeXPath (XPathNavigator n)
 	{
 		if (n.Matches ("/Type/Docs/param")) {
@@ -942,7 +981,7 @@ public class EcmaHelpSource : HelpSource {
 			case 2:
 				// Could either be a single member, or an overload thingy
 				try {
-					int dummy = int.Parse (nodes [1]); // is it an int
+					int.Parse (nodes [1]); // is it an int
 					
 					args.AddParam("show", "", "member");
 					args.AddParam("index", "", nodes [1]);
@@ -1053,6 +1092,13 @@ public class EcmaHelpSource : HelpSource {
 	public readonly ExtensionObject ExtObject;
 	public class ExtensionObject {
 		readonly EcmaHelpSource hs;
+
+		//
+		// We are setting this to quiet now, as we need to transition
+		// monodoc to run with the 2.x runtime and provide accurate
+		// information in those cases.
+		//
+		bool quiet = true;
 		
 		public ExtensionObject (EcmaHelpSource hs)
 		{
@@ -1148,9 +1194,11 @@ public class EcmaHelpSource : HelpSource {
 			return name.Replace("+", ".");
 		}
 
-
 		public string MonoImpInfo(string assemblyname, string typename, string membername, string arglist, bool strlong)
 		{
+			if (quiet)
+				return "";
+				
 			ArrayList a = new ArrayList();
 			if (arglist != "") a.Add(arglist);
 			return MonoImpInfo(assemblyname, typename, membername, a, strlong);
@@ -1158,6 +1206,9 @@ public class EcmaHelpSource : HelpSource {
 
 		public string MonoImpInfo(string assemblyname, string typename, string membername, XPathNodeIterator itr, bool strlong)
 		{
+			if (quiet)
+				return "";
+				
 			ArrayList rgs = new ArrayList ();
 			while (itr.MoveNext ())
 				rgs.Add (itr.Current.Value);
@@ -1172,7 +1223,10 @@ public class EcmaHelpSource : HelpSource {
 				
 				try {
 					assembly = Assembly.LoadWithPartialName(assemblyname);
-				} catch (Exception e) { }
+				} catch (Exception) {
+					// nothing.
+				}
+				
 				if (assembly == null) {
 					/*if (strlong) return "The assembly " + assemblyname + " is not available to MonoDoc.";
 					else return "";*/
@@ -1220,15 +1274,16 @@ public class EcmaHelpSource : HelpSource {
 					return MonoImpInfo(mi, "member", strlong);
 				}
 #endif
-				return "This member has not been implemented.";
-
-			} catch (Exception e) {
+			} catch (Exception) {
 				return "";
 			}
 		}
 		
 		public string MonoImpInfo(System.Reflection.MemberInfo mi, string itemtype, bool strlong)
 		{
+			if (quiet)
+				return "";
+				
 			string s = "";
 
 			object[] atts = mi.GetCustomAttributes(true);
@@ -1247,6 +1302,9 @@ public class EcmaHelpSource : HelpSource {
 
 		public string MonoImpInfo(string assemblyname, string typename, bool strlong)
 		{
+			if (quiet)
+				return "";
+				
 			try {
 				if (assemblyname == "")
 					return "";
@@ -1281,14 +1339,14 @@ public class EcmaHelpSource : HelpSource {
 
 				return s;
 
-			} catch (Exception e) {
+			} catch (Exception) {
 				return "";
 			}			
 		}
 
 		public bool MonoEditing ()
 		{
-			return SettingsHandler.Settings.EnableEditing && Settings.RunningGUI;
+			return SettingsHandler.Settings.EnableEditing;
 		}
 		
 		public bool IsToBeAdded(string text) {
@@ -1315,6 +1373,7 @@ public class EcmaHelpSource : HelpSource {
 		return fname;
 	}
 
+#if false
 	// This should have a little cache or something.
 	static XmlDocument GetDocument (HelpSource hs, string fname)
 	{
@@ -1330,7 +1389,8 @@ public class EcmaHelpSource : HelpSource {
 		
 		return doc;
 	}
-
+#endif
+	
 	string GetKindFromCaption (string s)
 	{
 		int p = s.LastIndexOf (' ');
@@ -1383,8 +1443,6 @@ public class EcmaHelpSource : HelpSource {
 			int idx = node.URL.IndexOf ('/');
 			return node.URL[idx+1] + ":" + full + "." + node.Caption;
 		}
-		return "";
-
 	}
 				
 	//
@@ -1749,7 +1807,7 @@ public class EcmaUncompiledHelpSource : EcmaHelpSource {
 			bool has_types = false;
 			foreach (XmlElement t in ns.SelectNodes("Type")) {
 				has_types = true;
-				string typename = t.GetAttribute("Name").Replace("+", ".");
+				string typename = EcmaDoc.GetDisplayName (t).Replace("+", ".");
 				
 				// Must load in each document to get the list of members...
 				XmlDocument typedoc = new XmlDocument();
@@ -1758,7 +1816,7 @@ public class EcmaUncompiledHelpSource : EcmaHelpSource {
 				
 				string url = ns.GetAttribute("Name") + "." + t.GetAttribute("Name");
 				Node typenode = nsnode.CreateNode(typename + " " + kind, "T:" + url);				
-				Node typemembers = typenode.CreateNode("Members", "T:" + url + "/*");
+				//Node typemembers = typenode.CreateNode("Members", "T:" + url + "/*");
 				
 				Hashtable groups = new Hashtable();
 				Hashtable groups_count = new Hashtable();
@@ -1828,11 +1886,15 @@ public class EcmaUncompiledHelpSource : EcmaHelpSource {
 			url = url.Substring(0, sidx);
 		}
 		
-		int pidx = url.LastIndexOf(".");
-		string ns = url.Substring(0, pidx);
-		string type = url.Substring(pidx+1).Replace(".", "+");
+		string ns, type;
+		if (!RootTree.GetNamespaceAndType (url, out ns, out type)) {
+			Console.Error.WriteLine ("Could not determine namespace/type for {0}",
+					url);
+			return null;
+		}
 		
-		string file = Path.Combine(Path.Combine(basedir.FullName, ns), type + ".xml");
+		string file = Path.Combine(Path.Combine(basedir.FullName, ns), 
+				ToEscapedTypeName (type).Replace ('.', '+') + ".xml");
 		if (!new FileInfo(file).Exists) return null;
 		
 		XmlDocument typedoc = new XmlDocument();
@@ -1855,7 +1917,7 @@ public class EcmaUncompiledHelpSource : EcmaHelpSource {
 			foreach (XmlNode node in nodes) {
 				XmlElement ns = summary.CreateElement ("namespace");
 				XmlAttribute attr = summary.CreateAttribute ("ns");
-				attr.Value = node.Attributes["Name"].Value;
+				attr.Value = EcmaDoc.GetDisplayName (node);
 				ns.Attributes.Append (attr);
 				elements.AppendChild (ns);
 			}
@@ -1899,7 +1961,7 @@ public class EcmaUncompiledHelpSource : EcmaHelpSource {
 			}
 			
 			XmlElement typenode = elements.CreateElement(typekind);
-			typenode.SetAttribute("name", t.GetAttribute("Name"));
+			typenode.SetAttribute("name", EcmaDoc.GetDisplayName (t).Replace ('+', '.'));
 			typenode.SetAttribute("fullname", ns + "." + t.GetAttribute("Name"));
 			typenode.AppendChild(elements.ImportNode(typedoc.SelectSingleNode("Type/Docs/summary"), true));
 			
